@@ -1,4 +1,6 @@
 import typing
+from importlib import import_module
+from warnings import warn
 
 from ._migration import getattr_migration
 from .version import VERSION
@@ -16,7 +18,6 @@ if typing.TYPE_CHECKING:
     )
 
     from . import dataclasses
-    from ._internal._generate_schema import GenerateSchema as GenerateSchema
     from .aliases import AliasChoices, AliasGenerator, AliasPath
     from .annotated_handlers import GetCoreSchemaHandler, GetJsonSchemaHandler
     from .config import ConfigDict, with_config
@@ -33,6 +34,7 @@ if typing.TYPE_CHECKING:
         AfterValidator,
         BeforeValidator,
         InstanceOf,
+        ModelWrapValidatorHandler,
         PlainValidator,
         SkipValidation,
         WrapValidator,
@@ -45,7 +47,13 @@ if typing.TYPE_CHECKING:
     from .type_adapter import TypeAdapter
     from .types import *
     from .validate_call_decorator import validate_call
-    from .warnings import PydanticDeprecatedSince20, PydanticDeprecatedSince26, PydanticDeprecationWarning
+    from .warnings import (
+        PydanticDeprecatedSince20,
+        PydanticDeprecatedSince26,
+        PydanticDeprecatedSince29,
+        PydanticDeprecationWarning,
+        PydanticExperimentalWarning,
+    )
 
     # this encourages pycharm to import `ValidationError` from here, not pydantic_core
     ValidationError = pydantic_core.ValidationError
@@ -67,6 +75,7 @@ __all__ = (
     'WrapValidator',
     'SkipValidation',
     'InstanceOf',
+    'ModelWrapValidatorHandler',
     # JSON Schema
     'WithJsonSchema',
     # deprecated V1 functional validators, these are imported via `__getattr__` below
@@ -128,6 +137,7 @@ __all__ = (
     'MySQLDsn',
     'MariaDBDsn',
     'ClickHouseDsn',
+    'SnowflakeDsn',
     'validate_email',
     # root_model
     'RootModel',
@@ -169,6 +179,7 @@ __all__ = (
     'Secret',
     'SecretStr',
     'SecretBytes',
+    'SocketPath',
     'StrictBool',
     'StrictBytes',
     'StrictInt',
@@ -194,6 +205,7 @@ __all__ = (
     'Tag',
     'Discriminator',
     'JsonValue',
+    'FailFast',
     # type_adapter
     'TypeAdapter',
     # version
@@ -202,12 +214,12 @@ __all__ = (
     # warnings
     'PydanticDeprecatedSince20',
     'PydanticDeprecatedSince26',
+    'PydanticDeprecatedSince29',
     'PydanticDeprecationWarning',
+    'PydanticExperimentalWarning',
     # annotated handlers
     'GetCoreSchemaHandler',
     'GetJsonSchemaHandler',
-    # generate schema from ._internal
-    'GenerateSchema',
     # pydantic_core
     'ValidationError',
     'ValidationInfo',
@@ -230,6 +242,7 @@ _dynamic_imports: 'dict[str, tuple[str, str]]' = {
     'WrapValidator': (__spec__.parent, '.functional_validators'),
     'SkipValidation': (__spec__.parent, '.functional_validators'),
     'InstanceOf': (__spec__.parent, '.functional_validators'),
+    'ModelWrapValidatorHandler': (__spec__.parent, '.functional_validators'),
     # JSON Schema
     'WithJsonSchema': (__spec__.parent, '.json_schema'),
     # functional serializers
@@ -285,6 +298,7 @@ _dynamic_imports: 'dict[str, tuple[str, str]]' = {
     'MySQLDsn': (__spec__.parent, '.networks'),
     'MariaDBDsn': (__spec__.parent, '.networks'),
     'ClickHouseDsn': (__spec__.parent, '.networks'),
+    'SnowflakeDsn': (__spec__.parent, '.networks'),
     'validate_email': (__spec__.parent, '.networks'),
     # root_model
     'RootModel': (__spec__.parent, '.root_model'),
@@ -329,6 +343,7 @@ _dynamic_imports: 'dict[str, tuple[str, str]]' = {
     'PaymentCardNumber': (__spec__.parent, '.types'),
     'ByteSize': (__spec__.parent, '.types'),
     'PastDate': (__spec__.parent, '.types'),
+    'SocketPath': (__spec__.parent, '.types'),
     'FutureDate': (__spec__.parent, '.types'),
     'PastDatetime': (__spec__.parent, '.types'),
     'FutureDatetime': (__spec__.parent, '.types'),
@@ -348,17 +363,18 @@ _dynamic_imports: 'dict[str, tuple[str, str]]' = {
     'Discriminator': (__spec__.parent, '.types'),
     'JsonValue': (__spec__.parent, '.types'),
     'OnErrorOmit': (__spec__.parent, '.types'),
+    'FailFast': (__spec__.parent, '.types'),
     # type_adapter
     'TypeAdapter': (__spec__.parent, '.type_adapter'),
     # warnings
     'PydanticDeprecatedSince20': (__spec__.parent, '.warnings'),
     'PydanticDeprecatedSince26': (__spec__.parent, '.warnings'),
+    'PydanticDeprecatedSince29': (__spec__.parent, '.warnings'),
     'PydanticDeprecationWarning': (__spec__.parent, '.warnings'),
+    'PydanticExperimentalWarning': (__spec__.parent, '.warnings'),
     # annotated handlers
     'GetCoreSchemaHandler': (__spec__.parent, '.annotated_handlers'),
     'GetJsonSchemaHandler': (__spec__.parent, '.annotated_handlers'),
-    # generate schema from ._internal
-    'GenerateSchema': (__spec__.parent, '._internal._generate_schema'),
     # pydantic_core stuff
     'ValidationError': ('pydantic_core', '.'),
     'ValidationInfo': ('pydantic_core', '.core_schema'),
@@ -374,26 +390,41 @@ _dynamic_imports: 'dict[str, tuple[str, str]]' = {
     'parse_obj_as': (__spec__.parent, '.deprecated.tools'),
     'schema_of': (__spec__.parent, '.deprecated.tools'),
     'schema_json_of': (__spec__.parent, '.deprecated.tools'),
+    # deprecated dynamic imports
     'FieldValidationInfo': ('pydantic_core', '.core_schema'),
+    'GenerateSchema': (__spec__.parent, '._internal._generate_schema'),
 }
+_deprecated_dynamic_imports = {'FieldValidationInfo', 'GenerateSchema'}
 
 _getattr_migration = getattr_migration(__name__)
 
 
 def __getattr__(attr_name: str) -> object:
+    if attr_name in _deprecated_dynamic_imports:
+        warn(
+            f'Importing {attr_name} from `pydantic` is deprecated. This feature is either no longer supported, or is not public.',
+            DeprecationWarning,
+            stacklevel=2,
+        )
+
     dynamic_attr = _dynamic_imports.get(attr_name)
     if dynamic_attr is None:
         return _getattr_migration(attr_name)
 
     package, module_name = dynamic_attr
 
-    from importlib import import_module
-
     if module_name == '__module__':
-        return import_module(f'.{attr_name}', package=package)
+        result = import_module(f'.{attr_name}', package=package)
+        globals()[attr_name] = result
+        return result
     else:
         module = import_module(module_name, package=package)
-        return getattr(module, attr_name)
+        result = getattr(module, attr_name)
+        g = globals()
+        for k, (_, v_module_name) in _dynamic_imports.items():
+            if v_module_name == module_name and k not in _deprecated_dynamic_imports:
+                g[k] = getattr(module, k)
+        return result
 
 
 def __dir__() -> 'list[str]':
