@@ -2,6 +2,7 @@ from __future__ import annotations as _annotations
 
 import warnings
 from contextlib import contextmanager
+from re import Pattern
 from typing import (
     TYPE_CHECKING,
     Any,
@@ -18,7 +19,7 @@ from typing_extensions import (
 from ..aliases import AliasGenerator
 from ..config import ConfigDict, ExtraValues, JsonDict, JsonEncoder, JsonSchemaExtraCallable
 from ..errors import PydanticUserError
-from ..warnings import PydanticDeprecatedSince20
+from ..warnings import PydanticDeprecatedSince20, PydanticDeprecatedSince210
 
 if not TYPE_CHECKING:
     # See PyCharm issues https://youtrack.jetbrains.com/issue/PY-21915
@@ -27,6 +28,7 @@ if not TYPE_CHECKING:
 
 if TYPE_CHECKING:
     from .._internal._schema_generation_shared import GenerateSchema
+    from ..fields import ComputedFieldInfo, FieldInfo
 
 DEPRECATION_MESSAGE = 'Support for class-based `config` is deprecated, use ConfigDict instead.'
 
@@ -57,6 +59,8 @@ class ConfigWrapper:
     # to construct error `loc`s, default `True`
     loc_by_alias: bool
     alias_generator: Callable[[str], str] | AliasGenerator | None
+    model_title_generator: Callable[[type], str] | None
+    field_title_generator: Callable[[str, FieldInfo | ComputedFieldInfo], str] | None
     ignored_types: tuple[type, ...]
     allow_inf_nan: bool
     json_schema_extra: JsonDict | JsonSchemaExtraCallable | None
@@ -67,12 +71,13 @@ class ConfigWrapper:
     # whether instances of models and dataclasses (including subclass instances) should re-validate, default 'never'
     revalidate_instances: Literal['always', 'never', 'subclass-instances']
     ser_json_timedelta: Literal['iso8601', 'float']
-    ser_json_bytes: Literal['utf8', 'base64']
-    ser_json_inf_nan: Literal['null', 'constants']
+    ser_json_bytes: Literal['utf8', 'base64', 'hex']
+    val_json_bytes: Literal['utf8', 'base64', 'hex']
+    ser_json_inf_nan: Literal['null', 'constants', 'strings']
     # whether to validate default values during validation, default False
     validate_default: bool
     validate_return: bool
-    protected_namespaces: tuple[str, ...]
+    protected_namespaces: tuple[str | Pattern[str], ...]
     hide_input_in_errors: bool
     defer_build: bool
     plugin_settings: dict[str, object] | None
@@ -118,7 +123,7 @@ class ConfigWrapper:
         config_dict_from_namespace = namespace.get('model_config')
 
         raw_annotations = namespace.get('__annotations__', {})
-        if raw_annotations.get('model_config') and not config_dict_from_namespace:
+        if raw_annotations.get('model_config') and config_dict_from_namespace is None:
             raise PydanticUserError(
                 '`model_config` cannot be used as a model field name. Use `model_config` for model configuration.',
                 code='model-config-invalid-field-name',
@@ -149,50 +154,53 @@ class ConfigWrapper:
                 except KeyError:
                     raise AttributeError(f'Config has no attribute {name!r}') from None
 
-    def core_config(self, obj: Any) -> core_schema.CoreConfig:
-        """Create a pydantic-core config, `obj` is just used to populate `title` if not set in config.
-
-        Pass `obj=None` if you do not want to attempt to infer the `title`.
+    def core_config(self, title: str | None) -> core_schema.CoreConfig:
+        """Create a pydantic-core config.
 
         We don't use getattr here since we don't want to populate with defaults.
 
         Args:
-            obj: An object used to populate `title` if not set in config.
+            title: The title to use if not set in config.
 
         Returns:
             A `CoreConfig` object created from config.
         """
+        config = self.config_dict
 
-        def dict_not_none(**kwargs: Any) -> Any:
-            return {k: v for k, v in kwargs.items() if v is not None}
-
-        core_config = core_schema.CoreConfig(
-            **dict_not_none(
-                title=self.config_dict.get('title') or (obj and obj.__name__),
-                extra_fields_behavior=self.config_dict.get('extra'),
-                allow_inf_nan=self.config_dict.get('allow_inf_nan'),
-                populate_by_name=self.config_dict.get('populate_by_name'),
-                str_strip_whitespace=self.config_dict.get('str_strip_whitespace'),
-                str_to_lower=self.config_dict.get('str_to_lower'),
-                str_to_upper=self.config_dict.get('str_to_upper'),
-                strict=self.config_dict.get('strict'),
-                ser_json_timedelta=self.config_dict.get('ser_json_timedelta'),
-                ser_json_bytes=self.config_dict.get('ser_json_bytes'),
-                ser_json_inf_nan=self.config_dict.get('ser_json_inf_nan'),
-                from_attributes=self.config_dict.get('from_attributes'),
-                loc_by_alias=self.config_dict.get('loc_by_alias'),
-                revalidate_instances=self.config_dict.get('revalidate_instances'),
-                validate_default=self.config_dict.get('validate_default'),
-                str_max_length=self.config_dict.get('str_max_length'),
-                str_min_length=self.config_dict.get('str_min_length'),
-                hide_input_in_errors=self.config_dict.get('hide_input_in_errors'),
-                coerce_numbers_to_str=self.config_dict.get('coerce_numbers_to_str'),
-                regex_engine=self.config_dict.get('regex_engine'),
-                validation_error_cause=self.config_dict.get('validation_error_cause'),
-                cache_strings=self.config_dict.get('cache_strings'),
+        if config.get('schema_generator') is not None:
+            warnings.warn(
+                'The `schema_generator` setting has been deprecated since v2.10. This setting no longer has any effect.',
+                PydanticDeprecatedSince210,
+                stacklevel=2,
             )
-        )
-        return core_config
+
+        core_config_values = {
+            'title': config.get('title') or title or None,
+            'extra_fields_behavior': config.get('extra'),
+            'allow_inf_nan': config.get('allow_inf_nan'),
+            'populate_by_name': config.get('populate_by_name'),
+            'str_strip_whitespace': config.get('str_strip_whitespace'),
+            'str_to_lower': config.get('str_to_lower'),
+            'str_to_upper': config.get('str_to_upper'),
+            'strict': config.get('strict'),
+            'ser_json_timedelta': config.get('ser_json_timedelta'),
+            'ser_json_bytes': config.get('ser_json_bytes'),
+            'val_json_bytes': config.get('val_json_bytes'),
+            'ser_json_inf_nan': config.get('ser_json_inf_nan'),
+            'from_attributes': config.get('from_attributes'),
+            'loc_by_alias': config.get('loc_by_alias'),
+            'revalidate_instances': config.get('revalidate_instances'),
+            'validate_default': config.get('validate_default'),
+            'str_max_length': config.get('str_max_length'),
+            'str_min_length': config.get('str_min_length'),
+            'hide_input_in_errors': config.get('hide_input_in_errors'),
+            'coerce_numbers_to_str': config.get('coerce_numbers_to_str'),
+            'regex_engine': config.get('regex_engine'),
+            'validation_error_cause': config.get('validation_error_cause'),
+            'cache_strings': config.get('cache_strings'),
+        }
+
+        return core_schema.CoreConfig(**{k: v for k, v in core_config_values.items() if v is not None})
 
     def __repr__(self):
         c = ', '.join(f'{k}={v!r}' for k, v in self.config_dict.items())
@@ -242,6 +250,8 @@ config_defaults = ConfigDict(
     from_attributes=False,
     loc_by_alias=True,
     alias_generator=None,
+    model_title_generator=None,
+    field_title_generator=None,
     ignored_types=(),
     allow_inf_nan=True,
     json_schema_extra=None,
@@ -249,15 +259,16 @@ config_defaults = ConfigDict(
     revalidate_instances='never',
     ser_json_timedelta='iso8601',
     ser_json_bytes='utf8',
+    val_json_bytes='utf8',
     ser_json_inf_nan='null',
     validate_default=False,
     validate_return=False,
-    protected_namespaces=('model_',),
+    protected_namespaces=('model_validate', 'model_dump'),
     hide_input_in_errors=False,
     json_encoders=None,
     defer_build=False,
-    plugin_settings=None,
     schema_generator=None,
+    plugin_settings=None,
     json_schema_serialization_defaults_required=False,
     json_schema_mode_override=None,
     coerce_numbers_to_str=False,
