@@ -271,28 +271,6 @@ inheriting2 = InheritingModel2(x=1, y='c')
 inheriting2.y = 'd'
 
 
-def _default_factory() -> str:
-    return 'x'
-
-
-test: List[str] = []
-
-
-class FieldDefaultTestingModel(BaseModel):
-    # Default
-    e: int = Field(None)
-    f: int = None
-
-    # Default factory
-    g: str = Field(default_factory=set)
-    h: int = Field(default_factory=_default_factory)
-    i: List[int] = Field(default_factory=list)
-    l_: str = Field(default_factory=3)
-
-    # Default and default factory
-    m: int = Field(default=1, default_factory=list)
-
-
 class ModelWithAnnotatedValidator(BaseModel):
     name: str
 
