@@ -1,19 +1,17 @@
+from dataclasses import InitVar
 from typing import Any, ClassVar, Generic, List, Optional, TypeVar, Union
 
-from pydantic import BaseModel, BaseSettings, Field, create_model, validator
+from typing_extensions import Self
+
+from pydantic import BaseModel, ConfigDict, Field, RootModel, create_model, field_validator, model_validator, validator
 from pydantic.dataclasses import dataclass
-from pydantic.generics import GenericModel
 
 
 class Model(BaseModel):
     x: float
     y: str
 
-    class Config:
-        orm_mode = True
-
-    class NotConfig:
-        allow_mutation = False
+    model_config = ConfigDict(from_attributes=True)
 
 
 class SelfReferencingModel(BaseModel):
@@ -24,21 +22,32 @@ class SelfReferencingModel(BaseModel):
         ...
 
 
-SelfReferencingModel.update_forward_refs()
+SelfReferencingModel.model_rebuild()
 
 model = Model(x=1, y='y')
 Model(x=1, y='y', z='z')
 model.x = 2
-model.from_orm(model)
+model.model_validate(model)
 
 self_referencing_model = SelfReferencingModel(submodel=SelfReferencingModel(submodel=None))
 
 
+class KwargsModel(BaseModel, from_attributes=True):
+    x: float
+    y: str
+
+
+kwargs_model = KwargsModel(x=1, y='y')
+KwargsModel(x=1, y='y', z='z')
+kwargs_model.x = 2
+kwargs_model.model_validate(kwargs_model.__dict__)
+
+
 class InheritingModel(Model):
     z: int = 1
 
 
-InheritingModel.from_orm(model)
+InheritingModel.model_validate(model.__dict__)
 
 
 class ForwardReferencingModel(Model):
@@ -49,7 +58,7 @@ class FutureModel(Model):
     pass
 
 
-ForwardReferencingModel.update_forward_refs()
+ForwardReferencingModel.model_rebuild()
 future_model = FutureModel(x=1, y='a')
 forward_model = ForwardReferencingModel(x=1, y='a', future=future_model)
 
@@ -57,27 +66,36 @@ forward_model = ForwardReferencingModel(x=1, y='a', future=future_model)
 class NoMutationModel(BaseModel):
     x: int
 
-    class Config:
-        allow_mutation = False
+    model_config = ConfigDict(frozen=True)
 
 
 class MutationModel(NoMutationModel):
-    a = 1
+    a: int = 1
 
-    class Config:
-        allow_mutation = True
-        orm_mode = True
+    model_config = ConfigDict(frozen=False, from_attributes=True)
 
 
 MutationModel(x=1).x = 2
-MutationModel.from_orm(model)
+MutationModel.model_validate(model.__dict__)
+
+
+class KwargsNoMutationModel(BaseModel, frozen=True):
+    x: int
+
+
+class KwargsMutationModel(KwargsNoMutationModel, frozen=False, from_attributes=True):
+    a: int = 1
+
+
+KwargsMutationModel(x=1).x = 2
+KwargsMutationModel.model_validate(model.__dict__)
 
 
 class OverrideModel(Model):
     x: int
 
 
-OverrideModel(x=1.5, y='b')
+OverrideModel(x=1, y='b')
 
 
 class Mixin:
@@ -108,11 +126,7 @@ class ClassVarModel(BaseModel):
 ClassVarModel(x=1)
 
 
-class Config:
-    validate_assignment = True
-
-
-@dataclass(config=Config)
+@dataclass(config={'validate_assignment': True})
 class AddProject:
     name: str
     slug: Optional[str]
@@ -145,40 +159,42 @@ dynamic_model.x = 2
 class FrozenModel(BaseModel):
     x: int
 
-    class Config:
-        frozen = True
+    model_config = ConfigDict(frozen=True)
 
 
 class NotFrozenModel(FrozenModel):
     a: int = 1
 
-    class Config:
-        frozen = False
-        orm_mode = True
+    model_config = ConfigDict(frozen=False, from_attributes=True)
 
 
 NotFrozenModel(x=1).x = 2
-NotFrozenModel.from_orm(model)
+NotFrozenModel.model_validate(model.__dict__)
+
+
+class KwargsFrozenModel(BaseModel, frozen=True):
+    x: int
+
+
+class KwargsNotFrozenModel(FrozenModel, frozen=False, from_attributes=True):
+    a: int = 1
+
+
+KwargsNotFrozenModel(x=1).x = 2
+KwargsNotFrozenModel.model_validate(model.__dict__)
 
 
 class ModelWithSelfField(BaseModel):
     self: str
 
 
-class SettingsModel(BaseSettings):
-    pass
-
-
-settings = SettingsModel.construct()
-
-
 def f(name: str) -> str:
     return name
 
 
 class ModelWithAllowReuseValidator(BaseModel):
     name: str
-    _normalize_name = validator('name', allow_reuse=True)(f)
+    normalize_name = field_validator('name')(f)
 
 
 model_with_allow_reuse_validator = ModelWithAllowReuseValidator(name='xyz')
@@ -187,7 +203,7 @@ model_with_allow_reuse_validator = ModelWithAllowReuseValidator(name='xyz')
 T = TypeVar('T')
 
 
-class Response(GenericModel, Generic[T]):
+class Response(BaseModel, Generic[T]):
     data: T
     error: Optional[str]
 
@@ -198,17 +214,17 @@ response = Response[Model](data=model, error=None)
 class ModelWithAnnotatedValidator(BaseModel):
     name: str
 
-    @validator('name')
+    @field_validator('name')
     def noop_validator_with_annotations(cls, name: str) -> str:
         return name
 
 
 def _default_factory_str() -> str:
-    ...
+    return 'x'
 
 
 def _default_factory_list() -> List[int]:
-    ...
+    return [1, 2, 3]
 
 
 class FieldDefaultTestingModel(BaseModel):
@@ -226,19 +242,6 @@ class FieldDefaultTestingModel(BaseModel):
     i: str = Field(default_factory=lambda: 'test')
 
 
-# Include the import down here to reduce the effect on line numbers
-from dataclasses import InitVar  # noqa E402
-
-
-@dataclass
-class MyDataClass:
-    foo: InitVar[str]
-    bar: str
-
-
-MyDataClass(foo='foo', bar='bar')
-
-
 _TModel = TypeVar('_TModel')
 _TType = TypeVar('_TType')
 
@@ -255,13 +258,13 @@ class OrmMixin(Generic[_TModel, _TType]):
         return cls.from_orm(model)
 
 
-class Sample(BaseModel):
-    foo: str
-    bar: Optional[str] = Field(description='optional')
-    zoo: Any
+@dataclass
+class MyDataClass:
+    foo: InitVar[str]
+    bar: str
 
 
-Sample(foo='hello world')
+MyDataClass(foo='foo', bar='bar')
 
 
 def get_my_custom_validator(field_name: str) -> Any:
@@ -275,6 +278,37 @@ def get_my_custom_validator(field_name: str) -> Any:
 def foo() -> None:
     class MyModel(BaseModel):
         number: int
-        custom_validator = get_my_custom_validator('number')
+        custom_validator = get_my_custom_validator('number')  # type: ignore[pydantic-field]
+
+        @model_validator(mode='before')
+        @classmethod
+        def validate_before(cls, values: Any) -> Any:
+            return values
+
+        @model_validator(mode='after')
+        def validate_after(self) -> Self:
+            return self
 
     MyModel(number=2)
+
+
+class InnerModel(BaseModel):
+    my_var: Union[str, None] = Field(default=None)
+
+
+class OuterModel(InnerModel):
+    pass
+
+
+m = OuterModel()
+if m.my_var is None:
+    # In https://github.com/pydantic/pydantic/issues/7399, this was unreachable
+    print('not unreachable')
+
+
+class Foo(BaseModel):
+    pass
+
+
+class Bar(Foo, RootModel[int]):
+    pass
