@@ -100,7 +100,7 @@ class DefaultTestingModel(BaseModel):
     # Required
     a: int
     b: int = ...
-# MYPY: error: Incompatible types in assignment (expression has type "ellipsis", variable has type "int")  [assignment]
+# MYPY: error: Incompatible types in assignment (expression has type "EllipsisType", variable has type "int")  [assignment]
     c: int = Field(...)
     d: Union[int, str]
     e = ...
@@ -303,34 +303,6 @@ inheriting2 = InheritingModel2(x=1, y='c')
 inheriting2.y = 'd'
 
 
-def _default_factory() -> str:
-    return 'x'
-
-
-test: List[str] = []
-
-
-class FieldDefaultTestingModel(BaseModel):
-    # Default
-    e: int = Field(None)
-# MYPY: error: Incompatible types in assignment (expression has type "None", variable has type "int")  [assignment]
-    f: int = None
-# MYPY: error: Incompatible types in assignment (expression has type "None", variable has type "int")  [assignment]
-
-    # Default factory
-    g: str = Field(default_factory=set)
-# MYPY: error: Incompatible types in assignment (expression has type "set[Any]", variable has type "str")  [assignment]
-    h: int = Field(default_factory=_default_factory)
-# MYPY: error: Incompatible types in assignment (expression has type "str", variable has type "int")  [assignment]
-    i: List[int] = Field(default_factory=list)
-    l_: str = Field(default_factory=3)
-# MYPY: error: Argument "default_factory" to "Field" has incompatible type "int"; expected "Callable[[], Any] | None"  [arg-type]
-
-    # Default and default factory
-    m: int = Field(default=1, default_factory=list)
-# MYPY: error: Field default and default_factory cannot be specified together  [pydantic-field]
-
-
 class ModelWithAnnotatedValidator(BaseModel):
     name: str
 
