@@ -1,6 +1,7 @@
 import datetime as dt
 import sys
 from dataclasses import dataclass
+from decimal import Decimal
 from typing import Any, Callable, Generic, Iterator, List, Optional, Set, TypeVar
 
 import pytest
@@ -644,3 +645,20 @@ def test_compatible_metadata_raises_correct_validation_error() -> None:
     ta = TypeAdapter(Annotated[str, BeforeValidator(lambda x: x), Field(pattern='abc')])
     with pytest.raises(ValidationError, match="String should match pattern 'abc'"):
         ta.validate_python('def')
+
+
+def test_decimal_constraints_after_annotation() -> None:
+    DecimalAnnotation = Annotated[Decimal, BeforeValidator(lambda v: v), Field(max_digits=10, decimal_places=4)]
+
+    ta = TypeAdapter(DecimalAnnotation)
+    assert ta.validate_python(Decimal('123.4567')) == Decimal('123.4567')
+
+    with pytest.raises(ValidationError) as e:
+        ta.validate_python(Decimal('123.45678'))
+
+    assert e.value.errors()[0]['type'] == 'decimal_max_places'
+
+    with pytest.raises(ValidationError) as e:
+        ta.validate_python(Decimal('12345678.901'))
+
+    assert e.value.errors()[0]['type'] == 'decimal_max_digits'
