@@ -1,6 +1,7 @@
 import random
 import sys
 from abc import ABC, abstractmethod
+from functools import cached_property, lru_cache, singledispatchmethod
 from typing import Any, Callable, ClassVar, Generic, List, Tuple, TypeVar
 
 import pytest
@@ -10,6 +11,7 @@ from typing_extensions import TypedDict
 from pydantic import (
     BaseModel,
     Field,
+    FieldSerializationInfo,
     GetCoreSchemaHandler,
     PrivateAttr,
     TypeAdapter,
@@ -21,13 +23,6 @@ from pydantic import (
 from pydantic.alias_generators import to_camel
 from pydantic.errors import PydanticUserError
 
-try:
-    from functools import cached_property, lru_cache, singledispatchmethod
-except ImportError:
-    cached_property = None
-    lru_cache = None
-    singledispatchmethod = None
-
 
 def test_computed_fields_get():
     class Rectangle(BaseModel):
@@ -180,7 +175,6 @@ def test_computed_fields_del():
     assert user.model_dump() == {'first': '', 'last': '', 'fullname': ' '}
 
 
-@pytest.mark.skipif(cached_property is None, reason='cached_property not available')
 def test_cached_property():
     class Model(BaseModel):
         minimum: int = Field(alias='min')
@@ -211,6 +205,13 @@ def test_cached_property():
     assert rect.model_dump(by_alias=True) == {'min': 10, 'max': 10_000, 'the magic number': first_n}
     assert rect.model_dump(by_alias=True, exclude={'random_number'}) == {'min': 10, 'max': 10000}
 
+    # `cached_property` is a non-data descriptor, assert that you can assign a value to it:
+    rect2 = Model(min=1, max=1)
+    rect2.cached_property_2 = 1
+    rect2._cached_property_3 = 2
+    assert rect2.cached_property_2 == 1
+    assert rect2._cached_property_3 == 2
+
 
 def test_properties_and_computed_fields():
     class Model(BaseModel):
@@ -256,7 +257,6 @@ def test_computed_fields_repr():
     assert repr(Model(x=2)) == 'Model(x=2, triple=6)'
 
 
-@pytest.mark.skipif(singledispatchmethod is None, reason='singledispatchmethod not available')
 def test_functools():
     class Model(BaseModel, frozen=True):
         x: int
@@ -798,3 +798,42 @@ def test_computed_field_excluded_from_model_dump_recursive() -> None:
 
     m = Model(bar=42)
     assert m.model_dump() == {'bar': 42, 'id': 'id: {"bar":42}'}
+
+
+def test_computed_field_with_field_serializer():
+    class MyModel(BaseModel):
+        other_field: int = 42
+
+        @computed_field
+        @property
+        def my_field(self) -> str:
+            return 'foo'
+
+        @field_serializer('*')
+        def my_field_serializer(self, value: Any, info: FieldSerializationInfo) -> Any:
+            return f'{info.field_name} = {value}'
+
+    assert MyModel().model_dump() == {'my_field': 'my_field = foo', 'other_field': 'other_field = 42'}
+
+
+def test_fields_on_instance_and_cls() -> None:
+    """For now, we support `model_fields` and `model_computed_fields` access on both instances and classes.
+
+    In V3, we should only support class access, though we need to preserve the current behavior for V2 compatibility."""
+
+    class Rectangle(BaseModel):
+        x: int
+        y: int
+
+        @computed_field
+        @property
+        def area(self) -> int:
+            return self.x * self.y
+
+    r = Rectangle(x=10, y=5)
+
+    for attr in {'model_fields', 'model_computed_fields'}:
+        assert getattr(r, attr) == getattr(Rectangle, attr)
+
+    assert set(r.model_fields) == {'x', 'y'}
+    assert set(r.model_computed_fields) == {'area'}
