@@ -558,14 +558,9 @@ def test_create_model_tuple_3():
 
 def test_create_model_protected_namespace_default():
     with pytest.warns(
-        UserWarning, match='Field "model_prefixed_field" in Model has conflict with protected namespace "model_"'
+        UserWarning, match='Field "model_dump_something" in Model has conflict with protected namespace "model_dump"'
     ):
-        create_model('Model', model_prefixed_field=(str, ...))
-
-
-def test_create_model_protected_namespace_real_conflict():
-    with pytest.raises(NameError, match='Field "model_dump" conflicts with member .* of protected namespace "model_"'):
-        create_model('Model', model_dump=(str, ...))
+        create_model('Model', model_dump_something=(str, ...))
 
 
 def test_create_model_custom_protected_namespace():
