@@ -16,7 +16,6 @@ from typing import Any
 from dirty_equals import HasRepr, IsInstance, IsStr
 
 from pydantic import BaseModel, ConfigDict
-from pydantic._internal._core_metadata import CoreMetadataHandler
 from pydantic._internal._core_utils import CoreSchemaOrField
 from pydantic.errors import PydanticInvalidForJsonSchema
 from pydantic.json_schema import GenerateJsonSchema, JsonSchemaValue
@@ -46,9 +45,7 @@ class FastAPIGenerateJsonSchema(GenerateJsonSchema):
     """
 
     def handle_invalid_for_json_schema(self, schema: CoreSchemaOrField, error_info: str) -> JsonSchemaValue:
-        # NOTE: I think it may be a good idea to rework this method to either not use CoreMetadataHandler,
-        #    and/or to make CoreMetadataHandler a public API.
-        if CoreMetadataHandler(schema).metadata.get('pydantic_js_modify_function') is not None:
+        if schema.get('metadata', {}).get('pydantic_js_modify_function') is not None:
             # Since there is a json schema modify function, assume that this type is meant to be handled,
             # and the modify function will set all properties as appropriate
             return {}
