@@ -1,10 +1,9 @@
-import sys
-from dataclasses import InitVar
+from typing import Union
 
 import pytest
 
 import pydantic.dataclasses
-from pydantic import BaseModel, Field, RootModel, ValidationError, fields
+from pydantic import BaseModel, ConfigDict, Field, PydanticUserError, RootModel, ValidationError, computed_field, fields
 
 
 def test_field_info_annotation_keyword_argument():
@@ -20,15 +19,18 @@ def test_field_info_annotation_keyword_argument():
     assert e.value.args == ('"annotation" is not permitted as a Field keyword argument',)
 
 
-@pytest.mark.skipif(sys.version_info >= (3, 8), reason='No error is thrown for `InitVar` for Python 3.8+')
-def test_init_var_does_not_work():
-    with pytest.raises(RuntimeError) as e:
+def test_field_info_annotated_attribute_name_clashing():
+    """This tests that `FieldInfo.from_annotated_attribute` will raise a `PydanticUserError` if attribute names clashes
+    with a type.
+    """
 
-        @pydantic.dataclasses.dataclass
-        class Model:
-            some_field: InitVar[str]
+    with pytest.raises(PydanticUserError):
 
-    assert e.value.args == ('InitVar is not supported in Python 3.7 as type information is lost',)
+        class SubModel(BaseModel):
+            a: int = 1
+
+        class Model(BaseModel):
+            SubModel: SubModel = Field()
 
 
 def test_init_var_field():
@@ -107,3 +109,64 @@ def test_frozen_field_repr():
 
     assert repr(Model.model_fields['non_frozen_field']) == 'FieldInfo(annotation=int, required=True)'
     assert repr(Model.model_fields['frozen_field']) == 'FieldInfo(annotation=int, required=True, frozen=True)'
+
+
+def test_model_field_default_info():
+    """Test that __repr_args__ of FieldInfo includes the default value when it's set to None."""
+
+    class Model(BaseModel):
+        a: Union[int, None] = Field(default=None)
+        b: Union[int, None] = None
+
+    assert str(Model.model_fields) == (
+        "{'a': FieldInfo(annotation=Union[int, NoneType], required=False, default=None), "
+        "'b': FieldInfo(annotation=Union[int, NoneType], required=False, default=None)}"
+    )
+
+
+def test_computed_field_raises_correct_attribute_error():
+    class Model(BaseModel):
+        model_config = ConfigDict(extra='allow')
+
+        @computed_field
+        def comp_field(self) -> str:
+            raise AttributeError('Computed field attribute error')
+
+        @property
+        def prop_field(self):
+            raise AttributeError('Property attribute error')
+
+    with pytest.raises(AttributeError, match='Computed field attribute error'):
+        Model().comp_field
+
+    with pytest.raises(AttributeError, match='Property attribute error'):
+        Model().prop_field
+
+    with pytest.raises(AttributeError, match=f"'{Model.__name__}' object has no attribute 'invalid_field'"):
+        Model().invalid_field
+
+
+@pytest.mark.parametrize('number', (1, 42, 443, 11.11, 0.553))
+def test_coerce_numbers_to_str_field_option(number):
+    class Model(BaseModel):
+        field: str = Field(coerce_numbers_to_str=True, max_length=10)
+
+    assert Model(field=number).field == str(number)
+
+
+@pytest.mark.parametrize('number', (1, 42, 443, 11.11, 0.553))
+def test_coerce_numbers_to_str_field_precedence(number):
+    class Model(BaseModel):
+        model_config = ConfigDict(coerce_numbers_to_str=True)
+
+        field: str = Field(coerce_numbers_to_str=False)
+
+    with pytest.raises(ValidationError):
+        Model(field=number)
+
+    class Model(BaseModel):
+        model_config = ConfigDict(coerce_numbers_to_str=False)
+
+        field: str = Field(coerce_numbers_to_str=True)
+
+    assert Model(field=number).field == str(number)
