@@ -21,6 +21,7 @@ from typing import (
     Mapping,
     NamedTuple,
     Optional,
+    OrderedDict,
     Sequence,
     Set,
     Tuple,
@@ -35,8 +36,8 @@ from pydantic_core import CoreSchema, core_schema
 from typing_extensions import (
     Annotated,
     Literal,
+    Never,
     NotRequired,
-    OrderedDict,
     ParamSpec,
     TypedDict,
     TypeVarTuple,
@@ -181,8 +182,7 @@ def test_methods_are_inherited():
 
 
 def test_config_is_inherited():
-    class CustomGenericModel(BaseModel, frozen=True):
-        ...
+    class CustomGenericModel(BaseModel, frozen=True): ...
 
     T = TypeVar('T')
 
@@ -359,7 +359,7 @@ def test_cache_keys_are_hashable(clean_cache):
     assert len(_GENERIC_TYPES_CACHE) == cache_size + 15
 
     class Model(BaseModel):
-        x: MyGenericModel[Callable[[C], Iterable[str]]] = Field(...)
+        x: MyGenericModel[Callable[[C], Iterable[str]]]
 
     models.append(Model)
     assert len(_GENERIC_TYPES_CACHE) == cache_size + 15
@@ -686,10 +686,7 @@ def test_nested():
     OuterT_SameType[int](i={'a': 8})
     OuterT_SameType[int](i=inner_int)
     OuterT_SameType[str](i=inner_str)
-    # TODO: The next line is failing, but passes in v1.
-    #   Should re-parse-from-dict if the pydantic_generic_origin is the same
-    # OuterT_SameType[str](i=inner_int_any)
-    OuterT_SameType[int](i=inner_int_any.model_dump())
+    OuterT_SameType[int](i=inner_int_any)
 
     with pytest.raises(ValidationError) as exc_info:
         OuterT_SameType[int](i=inner_str.model_dump())
@@ -707,11 +704,10 @@ def test_nested():
     # insert_assert(exc_info.value.errors(include_url=False))
     assert exc_info.value.errors(include_url=False) == [
         {
-            'type': 'model_type',
-            'loc': ('i',),
-            'msg': 'Input should be a valid dictionary or instance of InnerT[int]',
-            'input': InnerT[str](a='ate'),
-            'ctx': {'class_name': 'InnerT[int]'},
+            'type': 'int_parsing',
+            'loc': ('i', 'a'),
+            'msg': 'Input should be a valid integer, unable to parse string as an integer',
+            'input': 'ate',
         }
     ]
 
@@ -725,13 +721,7 @@ def test_nested():
         OuterT_SameType[int](i=inner_dict_any)
     # insert_assert(exc_info.value.errors(include_url=False))
     assert exc_info.value.errors(include_url=False) == [
-        {
-            'type': 'model_type',
-            'loc': ('i',),
-            'msg': 'Input should be a valid dictionary or instance of InnerT[int]',
-            'input': InnerT[Any](a={}),
-            'ctx': {'class_name': 'InnerT[int]'},
-        }
+        {'type': 'int_type', 'loc': ('i', 'a'), 'msg': 'Input should be a valid integer', 'input': {}}
     ]
 
 
@@ -993,8 +983,7 @@ def test_generic_model_redefined_without_cache_fail(create_module, monkeypatch):
         class MyGeneric(BaseModel, Generic[t]):
             value: t
 
-        class Model(BaseModel):
-            ...
+        class Model(BaseModel): ...
 
         concrete = MyGeneric[Model]
         _GENERIC_TYPES_CACHE.clear()
@@ -1506,8 +1495,7 @@ def test_generic_with_referenced_generic_type_bound():
     class ReferenceModel(BaseModel, Generic[T]):
         abstract_base_with_type: ModelWithType[T]
 
-    class MyInt(int):
-        ...
+    class MyInt(int): ...
 
     ReferenceModel[MyInt]
 
@@ -1518,11 +1506,9 @@ def test_generic_with_referenced_generic_union_type_bound():
     class ModelWithType(BaseModel, Generic[T]):
         some_type: Type[T]
 
-    class MyInt(int):
-        ...
+    class MyInt(int): ...
 
-    class MyStr(str):
-        ...
+    class MyStr(str): ...
 
     ModelWithType[MyInt]
     ModelWithType[MyStr]
@@ -1974,14 +1960,11 @@ def test_generic_annotated():
 def test_generic_subclass():
     T = TypeVar('T')
 
-    class A(BaseModel, Generic[T]):
-        ...
+    class A(BaseModel, Generic[T]): ...
 
-    class B(A[T], Generic[T]):
-        ...
+    class B(A[T], Generic[T]): ...
 
-    class C(B[T], Generic[T]):
-        ...
+    class C(B[T], Generic[T]): ...
 
     assert B[int].__name__ == 'B[int]'
     assert issubclass(B[int], B)
@@ -1993,11 +1976,9 @@ def test_generic_subclass_with_partial_application():
     T = TypeVar('T')
     S = TypeVar('S')
 
-    class A(BaseModel, Generic[T]):
-        ...
+    class A(BaseModel, Generic[T]): ...
 
-    class B(A[S], Generic[T, S]):
-        ...
+    class B(A[S], Generic[T, S]): ...
 
     PartiallyAppliedB = B[str, T]
     assert issubclass(PartiallyAppliedB[int], A)
@@ -2007,11 +1988,9 @@ def test_multilevel_generic_binding():
     T = TypeVar('T')
     S = TypeVar('S')
 
-    class A(BaseModel, Generic[T, S]):
-        ...
+    class A(BaseModel, Generic[T, S]): ...
 
-    class B(A[str, T], Generic[T]):
-        ...
+    class B(A[str, T], Generic[T]): ...
 
     assert B[int].__name__ == 'B[int]'
     assert issubclass(B[int], A)
@@ -2021,11 +2000,9 @@ def test_generic_subclass_with_extra_type():
     T = TypeVar('T')
     S = TypeVar('S')
 
-    class A(BaseModel, Generic[T]):
-        ...
+    class A(BaseModel, Generic[T]): ...
 
-    class B(A[S], Generic[T, S]):
-        ...
+    class B(A[S], Generic[T, S]): ...
 
     assert B[int, str].__name__ == 'B[int, str]', B[int, str].__name__
     assert issubclass(B[str, int], B)
@@ -2036,8 +2013,7 @@ def test_generic_subclass_with_extra_type_requires_all_params():
     T = TypeVar('T')
     S = TypeVar('S')
 
-    class A(BaseModel, Generic[T]):
-        ...
+    class A(BaseModel, Generic[T]): ...
 
     with pytest.raises(
         TypeError,
@@ -2046,8 +2022,7 @@ def test_generic_subclass_with_extra_type_requires_all_params():
         ),
     ):
 
-        class B(A[T], Generic[S]):
-            ...
+        class B(A[T], Generic[S]): ...
 
 
 def test_generic_subclass_with_extra_type_with_hint_message():
@@ -2076,21 +2051,17 @@ def test_generic_subclass_with_extra_type_with_hint_message():
             match='Classes should inherit from `BaseModel` before generic classes',
         ):
 
-            class ChildGenericClass(BaseGenericClass[E, Dict[str, Any]]):
-                ...
+            class ChildGenericClass(BaseGenericClass[E, Dict[str, Any]]): ...
 
 
 def test_multi_inheritance_generic_binding():
     T = TypeVar('T')
 
-    class A(BaseModel, Generic[T]):
-        ...
+    class A(BaseModel, Generic[T]): ...
 
-    class B(A[int], Generic[T]):
-        ...
+    class B(A[int], Generic[T]): ...
 
-    class C(B[str], Generic[T]):
-        ...
+    class C(B[str], Generic[T]): ...
 
     assert C[float].__name__ == 'C[float]'
     assert issubclass(C[float], B)
@@ -2183,13 +2154,14 @@ def test_parse_generic_json():
                 'type': 'object',
             }
         },
-        'properties': {'message': {'allOf': [{'$ref': '#/$defs/Payload'}], 'title': 'Message'}},
+        'properties': {'message': {'$ref': '#/$defs/Payload', 'title': 'Message'}},
         'required': ['message'],
         'title': 'MessageWrapper[test_parse_generic_json.<locals>.Payload]',
         'type': 'object',
     }
 
 
+@pytest.mark.skipif(sys.version_info > (3, 12), reason="memray doesn't yet support Python 3.13")
 def memray_limit_memory(limit):
     if '--memray' in sys.argv:
         return pytest.mark.limit_memory(limit)
@@ -2339,7 +2311,7 @@ def test_construct_generic_model_with_validation():
     class Page(BaseModel, Generic[T]):
         page: int = Field(ge=42)
         items: Sequence[T]
-        unenforced: PositiveInt = Field(..., lt=10)
+        unenforced: PositiveInt = Field(lt=10)
 
     with pytest.raises(ValidationError) as exc_info:
         Page[int](page=41, items=[], unenforced=11)
@@ -2433,8 +2405,9 @@ def test_generic_enum_bound():
         }
     ]
 
+    # insert_assert(Model[MyEnum].model_json_schema())
     assert Model[MyEnum].model_json_schema() == {
-        '$defs': {'MyEnum': {'const': 1, 'title': 'MyEnum'}},
+        '$defs': {'MyEnum': {'enum': [1], 'title': 'MyEnum', 'type': 'integer'}},
         'properties': {'x': {'$ref': '#/$defs/MyEnum'}},
         'required': ['x'],
         'title': 'Model[test_generic_enum_bound.<locals>.MyEnum]',
@@ -2484,8 +2457,9 @@ def test_generic_intenum_bound():
         }
     ]
 
+    # insert_assert(Model[MyEnum].model_json_schema())
     assert Model[MyEnum].model_json_schema() == {
-        '$defs': {'MyEnum': {'const': 1, 'title': 'MyEnum', 'type': 'integer'}},
+        '$defs': {'MyEnum': {'enum': [1], 'title': 'MyEnum', 'type': 'integer'}},
         'properties': {'x': {'$ref': '#/$defs/MyEnum'}},
         'required': ['x'],
         'title': 'Model[test_generic_intenum_bound.<locals>.MyEnum]',
@@ -2884,3 +2858,112 @@ def test_generic_with_not_required_in_typed_dict() -> None:
     ta_foo_generic = TypeAdapter(FooGeneric[str])
     assert ta_foo_generic.validate_python({'type': 'tomato'}) == {'type': 'tomato'}
     assert ta_foo_generic.validate_python({}) == {}
+
+
+def test_generic_with_allow_extra():
+    T = TypeVar('T')
+
+    # This used to raise an error related to accessing the __annotations__ attribute of the Generic class
+    class AllowExtraGeneric(BaseModel, Generic[T], extra='allow'):
+        data: T
+
+
+def test_generic_field():
+    """Test for https://github.com/pydantic/pydantic/issues/10039.
+
+    This was originally fixed by defining a custom MRO for Pydantic models,
+    but the fix from https://github.com/pydantic/pydantic/pull/10666 seemed
+    better. Test is still kept for historical purposes.
+    """
+
+    T = TypeVar('T')
+
+    class A(BaseModel, Generic[T]): ...
+
+    class B(A[T]): ...
+
+    class C(B[bool]): ...
+
+    class Model(BaseModel):
+        input_bool: A[bool]
+
+    Model(input_bool=C())
+
+
+def test_generic_any_or_never() -> None:
+    T = TypeVar('T')
+
+    class GenericModel(BaseModel, Generic[T]):
+        f: Union[T, int]
+
+    any_json_schema = GenericModel[Any].model_json_schema()
+    assert any_json_schema['properties']['f'] == {'title': 'F'}  # any type
+
+    never_json_schema = GenericModel[Never].model_json_schema()
+    assert never_json_schema['properties']['f'] == {'type': 'integer', 'title': 'F'}
+
+
+def test_revalidation_against_any() -> None:
+    T = TypeVar('T')
+
+    class ResponseModel(BaseModel, Generic[T]):
+        content: T
+
+    class Product(BaseModel):
+        name: str
+        price: float
+
+    class Order(BaseModel):
+        id: int
+        product: ResponseModel[Any]
+
+    product = Product(name='Apple', price=0.5)
+    response1: ResponseModel[Any] = ResponseModel[Any](content=product)
+    response2: ResponseModel[Any] = ResponseModel(content=product)
+    response3: ResponseModel[Any] = ResponseModel[Product](content=product)
+
+    for response in response1, response2, response3:
+        order = Order(id=1, product=response)
+        assert isinstance(order.product.content, Product)
+
+
+def test_revalidation_without_explicit_parametrization() -> None:
+    """Note, this is seen in the test above as well, but is added here for thoroughness."""
+
+    T1 = TypeVar('T1', bound=BaseModel)
+
+    class InnerModel(BaseModel, Generic[T1]):
+        model: T1
+
+    T2 = TypeVar('T2', bound=InnerModel)
+
+    class OuterModel(BaseModel, Generic[T2]):
+        inner: T2
+
+    class MyModel(BaseModel):
+        foo: int
+
+    # Construct two instances, with and without generic annotation in the constructor:
+    inner1 = InnerModel[MyModel](model=MyModel(foo=42))
+    inner2 = InnerModel(model=MyModel(foo=42))
+    assert inner1 == inner2
+
+    outer1 = OuterModel[InnerModel[MyModel]](inner=inner1)
+    outer2 = OuterModel[InnerModel[MyModel]](inner=inner2)
+    # implies that validation succeeds for both
+    assert outer1 == outer2
+
+
+def test_revalidation_with_basic_inference() -> None:
+    T = TypeVar('T')
+
+    class Inner(BaseModel, Generic[T]):
+        inner: T
+
+    class Holder(BaseModel, Generic[T]):
+        inner: Inner[T]
+
+    holder1 = Holder[int](inner=Inner[int](inner=1))
+    holder2 = Holder(inner=Inner(inner=1))
+    # implies that validation succeeds for both
+    assert holder1 == holder2
