@@ -2,7 +2,7 @@ import json
 from typing import Any, Union
 
 import pytest
-from pydantic_core import PydanticCustomError, Url
+from pydantic_core import PydanticCustomError, PydanticSerializationError, Url
 from typing_extensions import Annotated
 
 from pydantic import (
@@ -1180,3 +1180,24 @@ def test_max_length_base_multi_host() -> None:
 
     with pytest.raises(ValidationError, match=r'Value should have at most 45 items after validation'):
         Model(postgres='postgres://user:pass@localhost:5432/foobarbazfoo')
+
+
+def test_unexpected_ser() -> None:
+    ta = TypeAdapter(HttpUrl)
+    with pytest.raises(
+        PydanticSerializationError,
+        match="Expected `<class 'pydantic.networks.HttpUrl'>` but got `<class 'str'>` with value `'http://example.com'`",
+    ):
+        ta.dump_python('http://example.com', warnings='error')
+
+
+def test_url_ser() -> None:
+    ta = TypeAdapter(HttpUrl)
+    assert ta.dump_python(HttpUrl('http://example.com')) == HttpUrl('http://example.com')
+    assert ta.dump_json(HttpUrl('http://example.com')) == b'"http://example.com/"'
+
+
+def test_url_ser_as_any() -> None:
+    ta = TypeAdapter(Any)
+    assert ta.dump_python(HttpUrl('http://example.com')) == HttpUrl('http://example.com')
+    assert ta.dump_json(HttpUrl('http://example.com')) == b'"http://example.com/"'
