@@ -289,7 +289,7 @@ def test_private_attribute_multiple_inheritance():
 def test_private_attributes_not_dunder() -> None:
     with pytest.raises(
         NameError,
-        match="Private attributes must not use dunder names;" " use a single underscore prefix instead of '__foo__'.",
+        match='Private attributes must not use dunder names;' " use a single underscore prefix instead of '__foo__'.",
     ):
 
         class MyModel(BaseModel):
