@@ -1,18 +1,24 @@
 from datetime import datetime
 from typing import List, Optional
 
+import pytest
+
 from pydantic import BaseModel
 from pydantic.color import Color
 
 
-class User(BaseModel):
-    id: int
-    name: str = 'John Doe'
-    signup_ts: Optional[datetime] = None
-    friends: List[int] = []
+@pytest.fixture(scope='session', name='User')
+def user_fixture():
+    class User(BaseModel):
+        id: int
+        name: str = 'John Doe'
+        signup_ts: Optional[datetime] = None
+        friends: List[int] = []
+
+    return User
 
 
-def test_rich_repr() -> None:
+def test_rich_repr(User):
     user = User(id=22)
     rich_repr = list(user.__rich_repr__())
 
@@ -24,7 +30,8 @@ def test_rich_repr() -> None:
     ]
 
 
-def test_rich_repr_color() -> None:
+@pytest.mark.filterwarnings('ignore::DeprecationWarning')
+def test_rich_repr_color(User):
     color = Color((10, 20, 30, 0.1))
     rich_repr = list(color.__rich_repr__())
 
